//
//  MainViewController.m
//  iFreeThePostcode
//
//  Created by John McKerrell on 21/10/2008.
//  Copyright MKE Computing Ltd 2008. All rights reserved.
//

#import "MainViewController.h"
#import "MainView.h"
#import "iFreeThePostcodeAppDelegate.h"
#import <CFNetwork/CFNetwork.h>
#import <CFNetwork/CFHTTPMessage.h>

@implementation MainViewController

/**
 * Called when the Nib is initialised. We initialise
 * some variables here.
 */
- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil {
	if (self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil]) {
		// Custom initialization
	}
    /**
     * Are we submitting an update? NO
     */
    updating = NO;
    /**
     * Set our currentLocation to nil initially.
     */
    currentLocation = nil;
    /**
     * Set up the operations queue, we will use
     * this to submit the postcode in another thread.
     */
    queue = [[[NSOperationQueue alloc] init] retain];
    
    /**
     * Register for keyboard notifications,
     * I haven't got this working yet.
     */
    //[self registerForKeyboardNotifications];
	return self;
}

/**
 * This is called when the view is loaded, now
 * we can initialise elements of the view.
 */
- (void)viewDidLoad {
    /**
     * Retrieve any user defaults that have been saved.
     */
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    [userDefaults synchronize];
    /**
     * I'll probably put these back in actually, I removed
     * them as you will only need to submit a single postcode
     * once, but you might want to switch apps before submitting.
     */
    //[pcFirst setText: [userDefaults stringForKey:@"pcFirst"] ];
    //[pcSecond setText: [userDefaults stringForKey:@"pcSecond"] ];
    /**
     * Email address always loaded from defaults.
     */
    [emailAddress setText: [userDefaults stringForKey:@"emailAddress"] ];
    
    /**
     * Update the status of the submit button.
     */
    [self updateViewStatus];
    
    /**
     * Comment this out if you're developing in the simulator.
     */
    [testBtn setHidden:YES];

}

/**
 * This function is called when you click the
 * "Next" or "Done" button when editing a
 * text field. At this point we save the
 * entry in the user defaults. We also update
 * the status of the submit button.
 */
- (IBAction)textFieldUpdated:(id)sender {
    NSString *key;
    UITextField *field = (UITextField*)sender;
    /**
     * The name of the key in the defaults depends
     * on the text field that sent the message.
     */
    if( sender == pcFirst ) {
        key = @"pcFirst";
    } else if( sender == pcSecond ) {
        key = @"pcSecond";
    } else if( sender == emailAddress ) {
        key = @"emailAddress";
    }
    if( key ) {
        /**
         * Retrieve a reference to the user defaults object
         * and set the value.
         */
        NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
        [userDefaults setObject:[field text] forKey: key ];
        //NSLog(@"Storing user default: %@ for key: %@", [field text], key );
        //NSLog(@"Retrieving user default: %@ for key: %@", [userDefaults valueForKey:key], key );
    }
    
    /**
     * Update the status of the submit button, we might be
     * able to enable it now that the user has entered
     * some information.
     */
    [self updateViewStatus];
}

/**
 * Called when we begin editing a text field, we change the type of
 * the "Return" key depending on what's in the text fields.
 */
- (IBAction)textFieldBeganEditing:(id)sender {
    /**
     * We keep a reference to the currently active field.
     */
    NSLog(@"Setting activeField");
    activeField = sender;
    
    /**
     * Calculate the next field to be highlighted after
     * this one.
     */
    UITextField *nextField;
    if( sender == pcFirst ) {
        nextField = pcSecond;
    } else if ( sender == pcSecond ) {
        nextField = emailAddress;
    } else if ( sender == emailAddress ) {
        nextField = pcFirst;
    }
    //if( sender == pcFirst || sender == pcSecond ) {
        /**
        * If the next field has no text in it, then
        * we set the value of the "Return" key to "Next".
        * Otherwise we set it to "Done".
        */
        UITextField *textField = (UITextField*)sender;
        NSString *nextText = [nextField text];
        NSLog(@"Text field string is: %@", [textField text]);
        if( ! nextText || [nextText isEqualToString:@""] ) {
            [textField setReturnKeyType:UIReturnKeyNext];
            NSLog(@"Setting return key to Next");
        } else {
            [textField setReturnKeyType:UIReturnKeyDone];
            NSLog(@"Setting return key to Done");
        }
    //}
}


/**
 * This function is called when the "Return" key is pressed
 * on the virtual keyboard. If the "Return" key is set to
 * show "Done" we hide the keyboard, if it is set to "Next"
 * then we pass focus to the next field.
 */
- (IBAction)textFieldDoneEditing:(id)sender {
    NSLog( @"returnKeyType=%i UIReturnKeyNext=%i UIReturnKeyDone=%i", [sender returnKeyType], UIReturnKeyNext, UIReturnKeyDone );
    if( [sender returnKeyType] == UIReturnKeyNext ) {
        if( sender == pcFirst ) {
            [pcSecond becomeFirstResponder];
        } else if( sender == pcSecond ) {
            [emailAddress becomeFirstResponder];
        } else if( sender == emailAddress ) {
            [pcFirst becomeFirstResponder];
        }
    } else {
        [sender resignFirstResponder];
    }
}

/**
 * This function is attached to the buttons on the view and
 * is called when one of the buttons is clicked on.
 */
- (IBAction)buttonClicked:(id)sender {
    NSLog( @"Button clicked: %@", sender );
    if( sender == testBtn ) {
        /**
         * Generate a test location.
         */
        id appDelegate = [[UIApplication sharedApplication] delegate];
        [appDelegate testUpdateLocation];
    } else if( sender == linkBtn ) {
        /**
         * Link through to freethepostcode.org
         */
        id appDelegate = [[UIApplication sharedApplication] delegate];
        [appDelegate linkToSafari];
    } else if( sender == submitBtn && currentLocation ) {
        /**
         * Submit the postcode.
         */
         
        /**
         * Set the flag that says we're updating.
         */
        updating = YES;
        
        /**
         * Update the submit button's status so the user
         * can't click it again.
         */
        [self updateViewStatus];
        
        /**
         * Generate the submission URL.
         */
            //http://www.freethepostcode.org/submit?email=someone%40example.com&lat=1.234567890&lon=1.234567890&postcode1=SW1A&postcode2=0AA
        NSLog( @"currentLocation=%@", currentLocation );

        NSLog( @"lat=%+.6f lon=%+.6f", currentLocation.coordinate.latitude, currentLocation.coordinate.longitude );
        NSString *urlString = [NSString stringWithFormat:@"http://www.freethepostcode.org/submit?email=%@&lat=%.6f&lon=%.6f&postcode1=%@&postcode2=%@",
            [[emailAddress text] stringByAddingPercentEscapesUsingEncoding:NSUTF8StringEncoding],
            currentLocation.coordinate.latitude,
            currentLocation.coordinate.longitude,
            [[pcFirst text] stringByAddingPercentEscapesUsingEncoding:NSUTF8StringEncoding],
            [[pcSecond text] stringByAddingPercentEscapesUsingEncoding:NSUTF8StringEncoding] ];
        NSURL *url = [NSURL URLWithString:urlString];
        
        /**
         * Create a PostCodeSubmitOperation with the generated URL
         * and add it to our queue.
         */
        PostCodeSubmitOperation *plo = [[[PostCodeSubmitOperation alloc] initWithURL:url] autorelease];
        [queue addOperation:plo];
    }
}

/**
 * Called when we get a submission response. It gets passed the
 * full contents of the HTML page which is then parsed to find
 * out what happened.
 */
- (void)submissionResponse:(NSString*)page {
    if( page && [page rangeOfString:@"You should have an email on its way to confirm"].location != NSNotFound) {
        [[[UIAlertView alloc] initWithTitle:@"Success!" message:@"You should get an email to confirm shortly." delegate:nil cancelButtonTitle:@"Close" otherButtonTitles:nil] show];
    } else if( page ) {
        [[[UIAlertView alloc] initWithTitle:@"Submission Failed" message:@"We did not get a success message back from freethepostcode.org\nPlease make sure you have entered valid input." delegate:nil cancelButtonTitle:@"Close" otherButtonTitles:nil] show];
    } else {
        [[[UIAlertView alloc] initWithTitle:@"Submission Failed" message:@"There was a problem accessing freethepostcode.org" delegate:nil cancelButtonTitle:@"Close" otherButtonTitles:nil] show];
    }
    
    /**
     * Set the flag that says we're updating.
     */
    updating = NO;
    
    /**
     * Update the status of the submit button. It should now be
     * clickable if we have an accurate enough location.
     */
    [self updateViewStatus];
}


/**
 * Registers us to receive keyboard notifications,
 * not currently used.
 */
- (void)registerForKeyboardNotifications
{
    [[NSNotificationCenter defaultCenter] addObserver:self
            selector:@selector(keyboardWasShown:)
            name:UIKeyboardDidShowNotification object:nil];
 
    [[NSNotificationCenter defaultCenter] addObserver:self
            selector:@selector(keyboardWasHidden:)
            name:UIKeyboardDidHideNotification object:nil];
}

/**
 * Not currently used - supposed to scroll the view to show the keyboard.
 */
// Called when the UIKeyboardDidShowNotification is sent.
- (void)keyboardWasShown:(NSNotification*)aNotification
{
    if (keyboardShown)
        return;
 
    NSDictionary* info = [aNotification userInfo];
 
    // Get the size of the keyboard.
    NSValue* aValue = [info objectForKey:UIKeyboardBoundsUserInfoKey];
    CGSize keyboardSize = [aValue CGRectValue].size;
 
    // Resize the scroll view (which is the root view of the window)
    CGRect viewFrame = [scrollView frame];
    viewFrame.size.height -= keyboardSize.height;
    scrollView.frame = viewFrame;
    NSLog(@"Removing %f", keyboardSize.height);
 
    // Scroll the active text field into view.
    CGRect textFieldRect = [activeField frame];
    NSLog(@"Scrolling view: %@ to visible:  to fit field: %@", scrollView, activeField);
    [scrollView scrollRectToVisible:textFieldRect animated:YES];
 
    keyboardShown = YES;
}
 
/**
 * Not currently used - supposed to scroll the view to show the keyboard.
 */
// Called when the UIKeyboardDidHideNotification is sent
- (void)keyboardWasHidden:(NSNotification*)aNotification
{
    NSDictionary* info = [aNotification userInfo];
 
    // Get the size of the keyboard.
    NSValue* aValue = [info objectForKey:UIKeyboardBoundsUserInfoKey];
    CGSize keyboardSize = [aValue CGRectValue].size;
 
    // Reset the height of the scroll view to its original value
    CGRect viewFrame = [scrollView frame];
    viewFrame.size.height += keyboardSize.height;
    scrollView.frame = viewFrame;
 
    keyboardShown = NO;
}

/**
 * Called when the app delegate receives a new location.
 */
- (void)locationUpdated:(CLLocation *)newLocation {
    /**
     * If the user has locked the current location, ignore any updates.
     */
    if( locationLock.on )
        return;
    CLLocationAccuracy acc = [newLocation horizontalAccuracy];

    /**
     * Check that this location came within the last 5 minutes.
     */
    NSTimeInterval interval = [[newLocation timestamp] timeIntervalSinceNow];
    NSLog( @"timestamp=%f", interval );
    if( interval > 300 ) {
        return;
    }

    /**
     * Update the views to reflect the new location's information.
     */
    [latitude setText:[NSString stringWithFormat: @"%+.6f", newLocation.coordinate.latitude]];
    [longitude setText:[NSString stringWithFormat: @"%+.6f", newLocation.coordinate.longitude]];
    [accuracy setText:[NSString stringWithFormat:@"%.0f m", acc]];
    
    /**
     * Keep a reference to this location.
     */
    [currentLocation release];
    currentLocation = newLocation;
    [currentLocation retain];
    
    /**
     * Update the status of the submit button.
     */
    [self updateViewStatus];
}

/**
 * Updates the status of the view depending on the accuracy of the 
 * currently stored location. Updates the submit button, the timestamp
 * and the colour of the accuracy.
 * Also sets the activity control animating. 
 */
- (void)updateViewStatus{

    /**
     * This default accuracy is just used for testing,
     * should be overwritten in most cases though.
     */
    CLLocationAccuracy acc = 200;
    if( currentLocation ) {
        acc = [currentLocation horizontalAccuracy];
        NSLog( @"current location = %@", currentLocation );
    }
    
    /**
     * Updates the timestamp to display when the most recent location
     * was received.
     */
    NSDateFormatter *dateFormatter = [[[NSDateFormatter alloc] init] autorelease];
    [dateFormatter setDateStyle:NSDateFormatterMediumStyle];
    [dateFormatter setTimeStyle:NSDateFormatterMediumStyle];
    [lastUpdated setText:[[NSString alloc] initWithFormat:@"Last updated: %@", [dateFormatter stringFromDate: [currentLocation timestamp]]]];

    /**
     * If the text fields are not empty and we have an accurate
     * location, and we're not already submitting, then set the submit
     * button to be enabled, otherwise set it to be disabled.
     */
    //NSLog(@"acc=%f updating=%@", acc, updating);
    if( ! [[emailAddress text] isEqualToString:@""]
     && ! [[pcFirst text] isEqualToString:@""]
     && ! [[pcSecond text] isEqualToString:@""]
     && ! updating && acc < 100 ) {
        NSLog(@"Enabled");
        [submitBtn setEnabled:YES];
    } else {
        NSLog(@"Disabled");
        [submitBtn setEnabled:NO];
    }
    
    /**
     * Set the colour of the accuracy label to red or green.
     */
    if( acc < 100 ) {
        [accuracy setTextColor:[UIColor greenColor]];
    } else {
        [accuracy setTextColor:[UIColor redColor]];
    }
    
    /**
     * If we are updating (submitting) then show the activity control
     * and start it animating.
     */
    if( updating ) {
        [activity startAnimating];
        [activity setHidden:NO];
    } else {
        [activity setHidden:YES];
        [activity stopAnimating];
    }

}

/**
 * This app will only work in portrait mode.
 */
- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
	// Return YES for supported orientations
	return (interfaceOrientation == UIInterfaceOrientationPortrait);
}

/**
 * Not really much we can do if we receive a memory warning.
 */
- (void)didReceiveMemoryWarning {
	[super didReceiveMemoryWarning]; // Releases the view if it doesn't have a superview
	// Release anything that's not essential, such as cached data
}

/**
 * De-allocating, so we release the current location and the queue.
 */
- (void)dealloc {
    [currentLocation release];
    [queue release];
	[super dealloc];
}


@end
